<?php

define ('WPV_ITEMS_PER_PAGE', 10); // TODO move to constants.php, maybe use WPV_DEFAULT_LIST_ITEMS instead

/**
* Update messages for regular edit screens
*
* @param $messages
* @return $messages
*/

add_filter('post_updated_messages', 'wpv_post_updated_messages_filter', 9999);

function wpv_post_updated_messages_filter( $messages ) {
	global $post;

	$post_type = get_post_type();
	if ( $post_type == 'view' ) {
		$messages['view'] = array(
			0 => '', // Unused. Messages start at index 1.
			1 => __('View updated.', 'wpv-views'),
			2 => __('Custom field updated.'),
			3 => __('Custom field deleted.'),
			4 => __('View updated.', 'wpv-views'),
			/* translators: %s: date and time of the revision */
			5 => isset($_GET['revision']) ? sprintf( __('View restored to revision from %s', 'wpv-views'), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
			6 => __('View published.', 'wpv-views'),
			7 => __('View saved.', 'wpv-views'),
			8 => __('View submitted.', 'wpv-views'),
			9 => sprintf( __('View scheduled for: <strong>%1$s</strong>.', 'wpv-views'),
				// translators: Publish box date format, see http://php.net/date
				date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ) ),
			10 => __('View draft updated', 'wpv-views'),
			);
	}
	if ( $post_type == 'view-template' ) {
		$messages['view-template'] = array(
			0 => '', // Unused. Messages start at index 1.
			1 => __('Content template updated.', 'wpv-views'),
			2 => __('Custom field updated.'),
			3 => __('Custom field deleted.'),
			4 => __('Content template updated.', 'wpv-views'),
			/* translators: %s: date and time of the revision */
			5 => isset($_GET['revision']) ? sprintf( __('Content template restored to revision from %s', 'wpv-views'), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
			6 => __('Content template published.', 'wpv-views'),
			7 => __('Content template saved.', 'wpv-views'),
			8 => __('Content template submitted.', 'wpv-views'),
			9 => sprintf( __('Content template scheduled for: <strong>%1$s</strong>.', 'wpv-views'),
				// translators: Publish box date format, see http://php.net/date
				date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ) ),
			10 => __('Content template draft updated', 'wpv-views'),
			);
	}
	return $messages;
}

function wpv_render_checkboxes( $values, $selected, $name ) { // TODO only used in old Status Filter, safe to remove
	$checkboxes = '<ul>';
	foreach ( $values as $value ) {

		if ( in_array( $value, $selected ) ) {
			$checked = ' checked="checked"';
		} else {
			$checked = '';
		}
		$checkboxes .= '<li><label><input type="checkbox" name="_wpv_settings[' . $name . '][]" value="' . $value . '"' . $checked . ' />&nbsp;' . $value . '</label></li>';

	}
	$checkboxes .= '</ul>';

	return $checkboxes;
}

function wpv_render_filter_td( $row, $id, $name, $summary_function, $selected, $data ) { // TODO only used in old Status Filter, safe to remove

	$td = '<td><img src="' . WPV_URL . '/res/img/delete.png" onclick="on_delete_wpv_filter(\'' . $row . '\')" style="cursor: pointer" />';
	$td .= '<td class="wpv_td_filter">';
	$td .= "<div id=\"wpv-filter-" . $id . "-show\">\n";
	$td .= call_user_func($summary_function, $selected);
	$td .= "</div>\n";
	$td .= "<div id=\"wpv-filter-" . $id . "-edit\" style='background:" . WPV_EDIT_BACKGROUND . ";display:none'>\n";

	$td .= '<fieldset>';
	$td .= '<legend><strong>' . $name . ':</strong></legend>';
	$td .= '<div>' . $data . '</div>';
	$td .= '</fieldset>';
	ob_start();
	?>
		<input class="button-primary" type="button" value="<?php echo __('OK', 'wpv-views'); ?>" name="<?php echo __('OK', 'wpv-views'); ?>" onclick="wpv_show_filter_<?php echo $id; ?>_edit_ok()"/>
		<input class="button-secondary" type="button" value="<?php echo __('Cancel', 'wpv-views'); ?>" name="<?php echo __('Cancel', 'wpv-views'); ?>" onclick="wpv_show_filter_<?php echo $id; ?>_edit_cancel()"/>
	<?php
	$td .= ob_get_clean();
	$td .= '</div></td>';

	return $td;
}

// TODO move this AJAX calls to wpv-admin-ajax.php

/**
* Disable pagination result hint message
*/

add_action('wp_ajax_wpv_pagination_hint_result_disable', 'wpv_pagination_hint_result_disable_callback');

function wpv_pagination_hint_result_disable_callback() {
	$nonce = $_POST["wpnonce"];
	if (! wp_verify_nonce( $nonce, 'wpv_view_pagination_hint_result_dismiss_nonce' ) ) die("Security check");
	$user_ID = get_current_user_id();
	$user_help_setting = get_user_meta( $user_ID, 'wpv_view_editor_help_dismiss' );
	if ( isset( $user_help_setting[0]['pagination'] ) && $user_help_setting[0]['pagination'] == 'disable' ) {
		echo true;
	} else {
		$user_help_setting['pagination'] = 'disable';
		$result = update_user_meta( $user_ID, 'wpv_view_editor_help_dismiss', $user_help_setting );
		echo $result;
	}
	die();
}

/**
* Disable parametric search hint message TODO check if this is deprecated
*/

add_action('wp_ajax_wpv_parametric_hint_disable', 'wpv_parametric_hint_disable_callback');

function wpv_parametric_hint_disable_callback() {
	$nonce = $_POST["wpnonce"];
	if (! wp_verify_nonce( $nonce, 'wpv_view_parametric_hint_dismiss_nonce' ) ) die("Security check");
	$user_ID = get_current_user_id();
	$user_help_setting = get_user_meta( $user_ID, 'wpv_view_editor_help_dismiss' );
	if ( isset( $user_help_setting[0]['parametric_search'] ) && $user_help_setting[0]['parametric_search'] == 'disable' ) {
		echo true;
	} else {
		$user_help_setting['parametric_search'] = 'disable';
		$result = update_user_meta( $user_ID, 'wpv_view_editor_help_dismiss', $user_help_setting );
		echo $result;
	}
	die();
}

/**
* Disable inline Content Template hint message
*/

add_action('wp_ajax_wpv_content_template_hint_disable', 'wpv_content_template_hint_disable_callback');

function wpv_content_template_hint_disable_callback() {
	$nonce = $_POST["wpnonce"];
	if (! wp_verify_nonce( $nonce, 'wpv_view_content_template_hint_dismiss_nonce' ) ) die("Security check");
	$user_ID = get_current_user_id();
	$user_help_setting = get_user_meta( $user_ID, 'wpv_view_editor_help_dismiss' );
	if ( isset( $user_help_setting[0]['content_template'] ) && $user_help_setting[0]['content_template'] == 'disable' ) {
		echo true;
	} else {
		$user_help_setting['content_template'] = 'disable';
		$result = update_user_meta( $user_ID, 'wpv_view_editor_help_dismiss', $user_help_setting );
		echo $result;
	}
	die();
}

/**
* Disable Layout wizard hint message
*/

add_action('wp_ajax_wpv_layout_wizard_hint_disable', 'wpv_layout_wizard_hint_disable_callback');

function wpv_layout_wizard_hint_disable_callback() {
	$nonce = $_POST["wpnonce"];
	if (! wp_verify_nonce( $nonce, 'wpv_view_layout_wizard_hint_dismiss_nonce' ) ) die("Security check");
	$user_ID = get_current_user_id();
	$user_help_setting = get_user_meta( $user_ID, 'wpv_view_editor_help_dismiss' );
	if ( isset( $user_help_setting[0]['layout_wizard'] ) && $user_help_setting[0]['layout_wizard'] == 'disable' ) {
		echo true;
	} else {
		$user_help_setting['layout_wizard'] = 'disable';
		$result = update_user_meta( $user_ID, 'wpv_view_editor_help_dismiss', $user_help_setting );
		echo $result;
	}
	die();
}

/**
* Set default View settings and layout settings
*
* @param $settings field: view_settings or view_layout_settings
* @param $purpose purpose of the view: all, pagination, slide, parametric or full
* @return array() with desired values
*/

function wpv_view_defaults( $settings = 'view_settings', $purpose = 'full' ) {
	$defaults = array(
		// set the initial values for the View settings
		// Note: taxonomy_type is set in wpv-section-query-type.php to use the first available taxonomy
		'view_settings' => array(
			'view-query-mode'			=> 'normal',
			'view_description'			=> '',
			'view_purpose'				=> 'full',
			'query_type'				=> array('posts'),
			'taxonomy_type'				=> array('category'),
			'post_type_dont_include_current_page'	=> true,
			'taxonomy_hide_empty'			=> true,
			'taxonomy_include_non_empty_decendants'	=> true,
			'taxonomy_pad_counts'			=> true, // check this setting application
			'orderby'				=> 'post_date',
			'order'					=> 'DESC',
			'taxonomy_orderby'			=> 'name',
			'taxonomy_order'			=> 'DESC',
			'limit'					=> -1,
			'offset'				=> 0,
			'taxonomy_limit'			=> -1,
			'taxonomy_offset'			=> 0,
			'posts_per_page'			=> 10,
			'pagination'				=> array(
								'disable',
								'mode'				=> 'none',
								'preload_images'		=> true,
								'cache_pages'			=> true,
								'preload_pages'			=> true,
								'pre_reach'			=> 1,
								'page_selector_control_type'	=> 'drop_down',
								'spinner'			=> 'default',
								'spinner_image'			=> '',
								'spinner_image_uploaded'	=> '',
								), // this needs carefull review
			'ajax_pagination'			=> array(
								'disable',
								'style'				=> 'fade',
								'duration'			=> 500,
								),
			'rollover'				=> array(
								'preload_images'		=> true,
								'posts_per_page'		=> 1,
								'speed'				=> 5,
								'effect'			=> 'fade',
								'duration'			=> 500,
								),
			'filter_meta_html_state'		=> array(
								'html'				=> 'on',
								'css'				=> 'off',
								'js'				=> 'off',
								'img'				=> 'off',
								),
			'filter_meta_html'			=> "[wpv-filter-start hide=\"false\"]\n[wpv-filter-controls][/wpv-filter-controls]\n[wpv-filter-end]",
			'filter_meta_html_css'			=> '',
			'filter_meta_html_js'			=> '',
			'layout_meta_html_state'		=> array(
								'html'				=> 'on',
								'css'				=> 'off',
								'js'				=> 'off',
								'img'				=> 'off',
								),
			'layout_meta_html_css'			=> '',
			'layout_meta_html_js'			=> '',
		),
		'view_layout_settings' => array( // almost all of this settings are only needed to create the layout on the fly, so they are not needed here
			'additional_js'				=> '',
			'layout_meta_html'			=> "[wpv-layout-start]
	[wpv-items-found]
	<!-- wpv-loop-start -->
		<wpv-loop>
		</wpv-loop>
	<!-- wpv-loop-end -->
	[/wpv-items-found]
	[wpv-no-items-found]
		[wpml-string context=\"wpv-views\"]<strong>No items found</strong>[/wpml-string]
	[/wpv-no-items-found]
[wpv-layout-end]",
		),
	);
	switch( $purpose ) {
		case 'all':
			$defaults['view_settings']['sections-show-hide'] = array(
				'pagination'		=> 'off',
				'filter-extra'		=> 'off',
			);
			$defaults['view_settings']['view_purpose'] = 'all';
			break;
		case 'pagination':
			$defaults['view_settings']['pagination'][0] = 'enable';
			$defaults['view_settings']['pagination']['mode'] = 'paged';
			$defaults['view_settings']['sections-show-hide'] = array(
				'limit-offset'		=> 'off',
			);
			$defaults['view_settings']['view_purpose'] = 'pagination';
			break;
		case 'slider':
			$defaults['view_settings']['pagination'][0] = 'enable';
			$defaults['view_settings']['pagination']['mode'] = 'rollover';
			$defaults['view_settings']['sections-show-hide'] = array(
				'limit-offset'		=> 'off',
			);
			$defaults['view_settings']['view_purpose'] = 'slider';
			break;
		case 'parametric':
			$defaults['view_settings']['sections-show-hide'] = array(
				'pagination'		=> 'off',
			);
			$defaults['view_settings']['view_purpose'] = 'parametric';
			break;
		case 'full':
		default:
			$defaults['view_settings']['sections-show-hide'] = array(
			
			);
			$defaults['view_settings']['view_purpose'] = 'full';
			break;
	}
	return $defaults[$settings];
}

/**
* Set default WordPress Archives settings and layout settings
*
* @param $settings field: view_settings or view_layout_settings
* @return array() with desired values
*/

function wpv_wordpress_archives_defaults( $settings = 'view_settings' ) {
	$defaults = array(
		'view_settings' => array(
			'view-query-mode'			=> 'archive',
			'sections-show-hide'			=> array(
									'content'		=> 'off',
								)
		),
		'view_layout_settings' => array( // almost all of this settings are only needed to create the layout on the fly, so they are not needed here
			'additional_js'				=> '',
			'layout_meta_html'			=> "[wpv-layout-start]
	[wpv-items-found]
	<!-- wpv-loop-start -->
		<wpv-loop>
		</wpv-loop>
	<!-- wpv-loop-end -->
	[/wpv-items-found]
	[wpv-no-items-found]
		[wpml-string context=\"wpv-views\"]<strong>No posts found</strong>[/wpml-string]
	[/wpv-no-items-found]
[wpv-layout-end]",
		),
	);
	return $defaults[$settings];
}

/**
* Check the existence of a kind of post
*
* @param $query_mode kind of View object: normal or archive
* @return boolean
*/

function wpv_check_items_exists( $query_mode ) {
    global $wpdb;
    $query_mode = esc_sql( $query_mode );
    $q = 'SELECT DISTINCT ID FROM `' . $wpdb->prefix . 'posts` wpp
        WHERE
        ID in (SELECT post_id FROM `' . $wpdb->prefix . 'postmeta` WHERE `meta_value` like \'%view-query-mode%"' . $query_mode . '"%\' AND `meta_key`="_wpv_settings")
        AND
        wpp.post_status="publish" AND wpp.post_type="view"';

    $res = $wpdb->get_results( $q, OBJECT );
    return count( $res ) != 0;
}

/**
* Creates the query for listing Views and WordPress Archives
*
* @param $query_mode kind of View object: normal or archive
* @param $page number of page being displayed
* @param $search_term search term if exists
* @return elements rows
*/

function wpv_admin_menu_views_listing( $view_query_mode, $page = 1, $search_term = '' ) {
    global $wpdb;
    $view_query_mode = esc_sql( $view_query_mode );
    $search_addon = '';
    if ( !empty( $search_term ) ) {
        $search_term = esc_sql( $search_term );
        $search_addon = ' AND ( wpp.post_title LIKE "%' . $search_term . '%" OR ';
        $search_addon .= '(wpp.ID = wpmeta.post_id AND wpmeta.meta_key="_wpv_description" AND  wpmeta.meta_value LIKE "%' . $search_term . '%" )';
        $search_addon .= ' )';
    }

    $paging = '';
    if ( $view_query_mode == 'normal' ) {
        $page = (int) $page;
        $paging = ' LIMIT ' . (WPV_ITEMS_PER_PAGE*($page-1)) . ', ' . WPV_ITEMS_PER_PAGE . ' ';
    }

    $q = ('
        SELECT DISTINCT wpp.* FROM `' . $wpdb->prefix . 'posts` wpp, `' . $wpdb->prefix . 'postmeta` wpmeta
        WHERE
            ID in (SELECT post_id FROM `' . $wpdb->prefix . 'postmeta` WHERE `meta_value` like \'%view-query-mode%"' . $view_query_mode . '"%\' AND `meta_key`="_wpv_settings")
        AND
            wpp.post_status="publish"
        AND
            wpp.post_type="view"
            '. $search_addon .'
        ORDER BY wpp.post_title ASC
        ' . $paging . '
    ');
    //ORDER BY wpp.post_date DESC
    //ORDER BY wpp.post_date DESC

    $content = ''; 
    $views = $wpdb->get_results( $q, OBJECT );
    if ( !empty( $views ) ) {
        foreach ( $views as $view ) {
            switch ( $view_query_mode ) {
                case 'normal': $content .= wpv_admin_menu_views_listing_row( $view->ID ); break;
                case 'archive': $content .= wpv_admin_menu_archive_listing_row( $view->ID ); break;
            }
        }
    }

    return $content;
}

/**
* Creates the pagination for listing Views and WordPress Archives
*
* @param $query_mode kind of View object: normal or archive
* @param $page number of page being displayed
* @param $section kind of object this pagination is being applyed to: view or ct
* @param $search_term search term if exists
* @return elements rows
*/

function wpv_admin_menu_views_pager( $view_query_mode, $page = 1, $section = 'views', $search_term = '' ) {
    global $wpdb;
    $view_query_mode = esc_sql( $view_query_mode );
    $search_term = esc_sql( $search_term );
    $page = (int) $page;
    $query_add = $query_add2 = $search_addon = '';
    if ( $section == 'views' ) { // Query for views listing pagination
        if ( !empty( $search_term ) ) {
        $search_addon = ' AND ( wpp.post_title LIKE "%' . $search_term . '%"  ';
        $search_addon .= ' OR wpp.ID IN (select wpmeta.post_id from `' . $wpdb->prefix . 'postmeta` wpmeta where wpp.ID = wpmeta.post_id AND wpmeta.meta_key="_wpv_settings" AND wpmeta.meta_value LIKE "%' . $search_term . '%" )';
        $search_addon .= ' )';
        }  

        $q = ('
            SELECT DISTINCT count(wpp.ID) as views_count FROM `' . $wpdb->prefix . 'posts` wpp
            WHERE
                ID in (SELECT post_id FROM `'. $wpdb->prefix . 'postmeta` WHERE `meta_value` like \'%view-query-mode%"' . $view_query_mode . '"%\' AND `meta_key`="_wpv_settings")
            AND
                wpp.post_status="publish"
            AND
                wpp.post_type="view"
            ' . $search_addon . '
        ');
    }
    elseif ( $section == 'ct' ) {
        $join = $cond = '';
        if ( defined('ICL_LANGUAGE_CODE') ){    
            list($join, $cond) = WPV_template::_get_wpml_sql( 'view-template', ICL_LANGUAGE_CODE );
        }    
       //print_r($);
        //$posts = $wpdb->get_col( "SELECT {$wpdb->posts}.ID FROM {$wpdb->posts} {$join} WHERE post_type='{$type}' {$cond}" );
        if ( !empty( $search_term ) ){
            $query_add .= ' AND ( `' . $wpdb->prefix . 'posts`.post_title like "%' . $search_term . '%"';
            $query_add .= ' OR `' . $wpdb->prefix . 'posts`.ID IN (select wpmeta.post_id from `' . $wpdb->prefix . 'postmeta` wpmeta where `' . $wpdb->prefix . 'posts`.ID = wpmeta.post_id AND wpmeta.meta_key="_wpv-content-template-decription" AND wpmeta.meta_value LIKE "%' . $search_term . '%" )';
            $query_add .= ') ';
        }
        $q = ('
        SELECT DISTINCT count(`' . $wpdb->prefix . 'posts`.ID) as views_count FROM `' . $wpdb->prefix . 'posts` '.$join.'
        WHERE `' . $wpdb->prefix . 'posts`.post_status="publish" AND `' . $wpdb->prefix . 'posts`.post_type="view-template" 
        ' . $query_add . $cond );
    }
    if ( isset( $q ) ) {
        $views_count = $wpdb->get_var( $q );
    } else {
        $views_count = 0;
    }
    $content = '';
    
    if ( $views_count > 0 ) {
        $pages_count = ceil( $views_count / WPV_ITEMS_PER_PAGE );
        if ( $pages_count > 1 ){
        $content .= '<p>' . __('Go to page:','wpv-views') . '</p>';
        $content .= '<ul>';
        $content .= '<li>
                <a href="#" class="js-wpv-listing-pagination-nav wpv-filter-navigation-link js-wpv-listing-pagination-nav-prev hidden">&laquo; ' . __('Previous page','wpv-views') . '</a>
        </li>';

        for ( $i = 1; $i <= $pages_count; $i++ ) {
            $active = 'wpv-filter-navigation-link-inactive';
            if ( $page == $i )
                 $active = 'js-active active';
            $content .= '<li><a href="#" class="js-wpv-listing-pagination-nav ' . $active . '" data-page-num="' . $i . '" >' . $i . '</a></li>';
        }

        $content .= '<li>
                <a href="#" class="js-wpv-listing-pagination-nav wpv-filter-navigation-link js-wpv-listing-pagination-nav-next"> ' . __('Next page','wpv-views') . ' &raquo;</a>
        </li>';

        $content .= '</ul>';
        }
        //return $pages_count;
    }

    return $content;
}

/**
* Cleans the WordPress Media popup to be used in Views and WordPress Archives
*
* @param $strings elements to be included
* @return $strings without the unwanted sections
*/

add_filter( 'media_view_strings', 'custom_media_uploader' );

function custom_media_uploader( $strings ) {
	if ( isset( $_GET['page'] ) && ( 'view-archives-editor' == $_GET['page'] || 'views-editor' == $_GET['page'] ) ) {
		unset( $strings['createGalleryTitle'] ); //Create Gallery
	}
	return $strings;
}

/**
* Add View button to codemirror editor
*
* @param $editor_id ID for the relevant textarea, to be set as active editor
* @param $inline TODO document this
* @return $strings without the unwanted sections
*/

function wpv_add_v_icon_to_codemirror( $editor_id, $inline = false ) {
    
    global $WP_Views;
    $view = '';
    if ( isset($_GET['view_id']) ){
        $view = $_GET['view_id'];
    }
    $is_taxonomy = false;
    $post_hidden = '';
    $tax_hidden = ' hidden';
    $meta = get_post_meta( $view, '_wpv_settings', true);
    
    if ( isset($meta['query_type']) && $meta['query_type'][0] == 'taxonomy'){
           $is_taxonomy = true;
           $post_hidden = ' hidden';
           $tax_hidden = '';
    }
    
    
    
    $WP_Views->editor_addon = new Editor_addon('wpv-views', 
            __('Insert Views Shortcodes', 'wpv-views'), 
            WPV_URL . '/res/js/views_editor_plugin.js', 
            WPV_URL . '/res/img/bw_icon16.png');

    if ( !$inline ){ echo '<div class="wpv-vicon-for-posts'. $post_hidden .'">';}
    
    add_short_codes_to_js( array('post', 'taxonomy', 'post-view', 'view-form'), $WP_Views->editor_addon );
    $WP_Views->editor_addon->add_form_button('', $editor_id , true, true, true);
    if ( !$inline ){echo '</div>';  }
    
    if ( !$inline ){
        echo '<div class="wpv-vicon-for-taxonomy'. $tax_hidden .'">';
        remove_filter('editor_addon_menus_wpv-views', 'wpv_post_taxonomies_editor_addon_menus_wpv_views_filter', 11);
        add_filter('editor_addon_menus_wpv-views', 'wpv_layout_taxonomy_V');
    
        $WP_Views->editor_addon->add_form_button('', $editor_id, true, true, true);
                            
        remove_filter('editor_addon_menus_wpv-views', 'wpv_layout_taxonomy_V');
        add_filter('editor_addon_menus_wpv-views', 'wpv_post_taxonomies_editor_addon_menus_wpv_views_filter', 11);
        echo '</div>';
    }
}

/**
* Add CRED button to codemirror editor
*
* @param $editor_id ID for the relevant textarea, to be set as active editor
* @return $strings without the unwanted sections
*/

function wpv_add_cred_to_codemirror( $editor_id ){
      echo apply_filters('wpv_meta_html_add_form_button', '', '#'.$editor_id);
}

//Update defaults and create new CT for slider view
function wpv_create_new_ct_for_slider_view($id, $view_title){
         global $wpdb;
     $i = 0; $add = '';
     
     while ( $i == 0 ){
         
        $ct_name = $view_title.' - slide'.$add;
        $total = $wpdb->get_var( $wpdb->prepare(
        'SELECT count(ID) FROM ' . $wpdb->posts . ' WHERE post_title = %s AND post_type=\'view-template\'',
        $ct_name
        ));
       
        $add++;
        if ( $total <= 0){
            $i=1;
        }
     }    
    
     $new_template = array(
          'post_title'    => $ct_name,
          'post_type'      => 'view-template',
          'post_content'  => '[wpv-post-link]',
          'post_status'   => 'publish',
          'post_author'   => 1
     );

     $post_id = wp_insert_post( $new_template );
     update_post_meta( $post_id, '_wpv_view_template_mode', 'raw_mode');
     update_post_meta( $post_id, '_wpv-content-template-decription', '');
     return $post_id;
}
